"""Input widget to select a color"""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2022-2023
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.
import ipyvuetify as v

try:
    from . import settings
except:
    import settings


class colorPicker():
    """
    Input widget to select a color.

    Parameters
    ----------
    color : str, optional
        Initial color selected on the widget expressed in hexadecimal format '#RRGGBB' (default is '#FF0000')
    dark : bool, optional
        If True, the popup color selection will have a dark background (default is settings.dark_mode)
    width : int, optional
        Width of the widget in pixels (default is 40)
    height : int, optional
        Height of the widget in pixels (default is 30)
    rounded : bool, optional
        If True the color widget is displayed as a round button (default is False)
    canvas_height : int, optional
        Height of the canvas displayed on top of the popup window to select the colors (default is True)
    show_canvas : bool, optional
        If True the popup window will show the color canvas (default is True)
    show_mode_switch : bool, optional
        If True the popup window will show mode switch control among RGB, HSL and HAX (default is True)
    show_inputs : bool, optional
        If True the popup window will show the input field for the color components (default is True)
    show_swatches : bool, optional
        If True the popup window will show the color swatches (default is True)
    swatches_max_height : int, optional
        Height in pixels of the swatches area in the popup window (default is 164)
    text : str, optional
        Text to display in the color button (default is '')
    textweight : int, optional
        Weight of the text to be shown in the button (default is 400, Bold is any value greater or equal to 500)
    onchange : function, optional
        Python function to call when the user selects a different color. The function will receive no parameters. (default is None)
    offset_x : bool, optional
        If True the popup window will be opened on the right of the color button (default is False)
    offset_y : bool, optional
        If True the popup window will be opened on the bottom of the color button (default is True)
    disabled : bool, optional
        True if the selection of the color is disabled, False otherwise (default is False)

    Example
    -------
    Creation of a color picker widget to select a color::
        
        from vois.vuetify import colorPicker
        from ipywidgets import widgets
        from IPython.display import display

        output = widgets.Output()

        def onchange():
            with output:
                print('Changed to', c.color)

        c = colorPicker.colorPicker(color='#00AAFF',
                                    width=30, height=30,
                                    rounded=False, 
                                    onchange=onchange,
                                    offset_x=True,
                                    offset_y=False)

        display(c.draw())
        display(output)

    .. figure:: figures/colorPicker.png
       :scale: 100 %
       :alt: colorPicker widget

       Example of a colorPicker to select a color
    """
    
    def hex2rgb(self, color):
        if color[0] == '#':
            color = color[1:]
        rgb = (int(color[0:2],16), int(color[2:4],16), int(color[4:6],16))
        return rgb

    def textDark(self, color):
        r,g,b = self.hex2rgb(color)
        if r+g+b <= 255+128: return True
        return False
    
    
    def __init__(self, color="#FF0000", dark=settings.dark_mode, width=40, height=30, rounded=False, canvas_height=100,
                       show_canvas=True, show_mode_switch=True, show_inputs=True, show_swatches=True,
                       swatches_max_height=164, text='', textweight=400, onchange=None,
                       offset_x=False, offset_y=True,
                       disabled=False):
        
        self._color = str(color).upper()
        self.dark = dark
        self.width   = width
        self.height  = height
        self.rounded = rounded
        self.canvas_height       = canvas_height
        self.show_canvas         = show_canvas
        self.show_mode_switch    = show_mode_switch
        self.show_inputs         = show_inputs
        self.show_swatches       = show_swatches
        self.swatches_max_height = swatches_max_height
        self.text       = text
        self.textweight = textweight
        self.onchange   = onchange
        self.offset_x   = offset_x
        self.offset_y   = offset_y
        self._disabled  = disabled

        if self._disabled: von = ''
        else:              von = 'menuData.on'
        self.button = v.Btn(v_on=von, depressed=True, large=False, dense=True, class_='pa-0 ma-0', color=self._color,
                            rounded=self.rounded, height=self.height, width=self.width, min_width=self.width, dark=self.textDark(self._color),
                            style_='font-weight: %d; text-transform: none' % self.textweight, children=[self.text])

        self.p = v.ColorPicker(value=self._color, flat=True, class_="pa-0 ma-0", style_="min-width: 300px;",
                               canvas_height=self.canvas_height, show_swatches=self.show_swatches, dark=self.dark,
                               swatches_max_height=self.swatches_max_height, hide_canvas=not self.show_canvas,
                               hide_mode_switch=not self.show_mode_switch, hide_inputs=not self.show_inputs)
        self.p.on_event('input', self.__internal_onchange)


        self.m = v.Menu(offset_x=self.offset_x, offset_y=self.offset_y, open_on_hover=False, dense=True, close_on_click=True, close_on_content_click=False,
                        v_slots=[{'name': 'activator', 'variable': 'menuData', 'children': self.button}],
                        children=[self.p])

        
    # Manage 'input' event
    def __internal_onchange(self, widget, event, data):
        if data.upper() != self._color.upper():
            self._color = data.upper()
            self.button.color = self._color
            self.button.dark  = self.textDark(self._color)
            if self.onchange:
                self.onchange()
        
        
    # Returns the vuetify object to display (the v.Menu)
    def draw(self):
        """Returns the ipyvuetify object to display (the internal v.Menu)"""
        return self.m

    
    # color property
    @property
    def color(self):
        """
        Get/Set the selected color.
        
        Returns
        --------
        c : str
            color currently selected

        Example
        -------
        Programmatically change the color::
            
            picker.color = '#00FF00'
            print(picker.color)
        
        """
        return self._color

    
    @color.setter
    def color(self, c):
        if isinstance(c, str):
            self._color = c
            self.p.value = self._color
            self.button.color = self._color
            self.button.dark  = self.textDark(self._color)
            if self.onchange:
                self.onchange()

                
    # disabled property
    @property
    def disabled(self):
        """
        Get/Set the disabled state of the widget.
        
        Returns
        --------
        flag : bool
            True if the widget is disabled, False otherwise

        Example
        -------
        Programmatically change the date::
            
            picker.disabled = True
            print(picker.disabled)
        
        """
        return self._disabled

    
    @disabled.setter
    def disabled(self, flag):
        self._disabled = bool(flag)
        if self._disabled: von = ''
        else:              von = 'menuData.on'
        self.button.v_on = von
                


"""Slider widget is a better visualization of the number input. It is used for gathering numerical user data."""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2022-2023
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.
from IPython.display import display
import ipyvuetify as v

try:
    from . import settings
except:
    import settings


#####################################################################################################################################################
# Integer slider class
#####################################################################################################################################################
class slider:
    """
    Slider widget is a better visualization of the number input. It is used for gathering numerical user data.
        
    Parameters
    ----------
    selectedvalue : numeric
        Initial value of the slider
    minvalue : numeric
        Minimal value selectable by the user
    maxvalue : numeric
        Maximum value selectable by the user
    step : numeric, optional
        Step interval for ticks (default is 1.0)
    vertical : bool, optional
        Flag that controls the direction of the slider: horizontal or vertical (default is False)
    color : str, optional
        Color used for the widget (default is the color_first defined in the settings.py module)
    onchange : function, optional
        Python function to call when the user selects a value. The function will receive a parameter of numeric type containing the current value of the slider widget
    height : int, optional
        Height of the slider widget in pixel (default is 120 pixels)
    width : int, optional
        Width of the slider widget in pixel. It is needed only for vertical sliders (default is None)
            
    Example
    -------
    Creation and display of a slider widget::
        
        from vois.vuetify import slider
        from ipywidgets import widgets
        from IPython.display import display

        output = widgets.Output()
        display(output)

        def onchange(value):
            with output:
                print(value)

        s = slider.slider(2015, 2010,2021, onchange=onchange)
        display(s.draw())

    .. figure:: figures/slider.png
       :scale: 100 %
       :alt: slider widget

       Slider widget example
   """

   
    # Initialization
    def __init__(self, selectedvalue, minvalue, maxvalue, vertical=False, color=settings.color_first, onchange=None, height=120, width=None, step=1.0):
        
        self.minvalue = minvalue
        self.maxvalue = maxvalue
        self.step     = step
        self.vertical = vertical
        self.width    = width
        self.onchange = onchange
        
        if vertical: c = "pa-0 ma-0 ml-n12 mr-n12 mt-n9 mb-n10"
        else:        c = "pa-0 ma-0 ml-5 mr-5 mt-n2 mb-n12"
        self.slider = v.Slider(v_model=selectedvalue, dense=True, small=True, thumb_color=color, 
                               thumb_label="always", thumb_size=32, ticks=True, ticks_size=10, 
                               color=color, track_color="grey", class_=c,
                               min=self.minvalue, max=self.maxvalue, step=self.step,
                               vertical=self.vertical, height=height)
        
        # If requested onchange management
        if not self.onchange is None:
            self.slider.on_event('end', self.__internal_onchange)

    # Get the value
    @property
    def value(self):
        """
        Get/Set the current value.
        
        Returns
        --------
        value : numeric
            Current value selected

        Example
        -------
        Programmatically set the value and print it::
            
            s.value = 2012
            print(s.value)
        
        """
        return self.slider.v_model
        
    # Set the value
    @value.setter
    def value(self, v):
        if v >= self.minvalue and v <= self.maxvalue:
            self.slider.v_model = v
            if self.onchange:
                self.onchange(self.slider.v_model)
    
    
    # Manage onchange event
    def __internal_onchange(self, widget=None, event=None, data=None):
        if self.onchange:
            self.onchange(data)
    
    
    # Returns the vuetify object to display (the v.Container)
    def draw(self):
        """Returns the ipyvuetify object to display (the internal v.Html that contains a v.Slider widget as its only child)"""
        if self.vertical and not self.width is None:
            return v.Html(tag='div',children=[self.slider], style_='width: %dpx; overflow: hidden;' % self.width)
        else:
            return v.Html(tag='div',children=[self.slider], style_='overflow: hidden;')
            



"""Button widget to call a python function when clicked."""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2022-2023
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.
from IPython.display import display
import ipyvuetify as v

try:
    from . import settings
    from . import fontsettings
except:
    import settings
    import fontsettings


#####################################################################################################################################################
# Button class. On click python function called when clicked
# Uses settings font to display the button text and interactivity when hover
#####################################################################################################################################################
class button():
    """
    Button widget to call a python function when clicked.
        
    Parameters
    ----------
    text : str
        Test string to be displayed on the button widget
    onclick : function, optional
        Python function to call when the user clicks on the button. The function will receive as parameter the value of the argument (default is None)
    argument : any, optional
        Argument to be passed to the onclick function when user click on the label (default is None)
    width : int, optional
        Wuidth of the button widget in pixels (default is 20)
    height : int, optional
        Height of the button widget in pixels (default is 20)
    selected : bool, optional
        Flag to show the button as selected (default is False)
    disabled : bool, optional
        Flag to show the button as disabled (default is False)
    tooltip : str, optional
        Tooltip text to show when the user hovers on the button (default is '')
    large : bool, optional
        Flag that sets the large version of the button (default is False)
    small : bool, optional
        Flag that sets the small version of the button (default is False)
    xsmall : bool, optional
        Flag that sets the xsmall version of the button (default is False)
    outlined : bool, optional
        Flag to show the button as outlined (default is False)
    textweight : int, optional
        Weight of the text to be shown in the label (default is 500, Bold is any value greater or equal to 500)
    href : str, optional
        URL to open when the button is clicked (default is None)
    target : str, optional
        Designates the target attribute (where the URL page is opened, for instance: '_blank' to open it in a new browser tab). This should only be applied when using the href parameter (default is None)
    onlytext : bool, optional
         If True, the button will contain only the text (default is False)
    textcolor : str, optional
        Color used for the button text (default is None)
    icon: str, optional
        Name of the icon to display aside the text of the label (default is None)
    iconlarge : bool, optional
        Flag that sets the large version of the icon (default is False)
    iconsmall : bool, optional
        Flag that sets the small version of the icon (default is False)
    iconleft : bool, optional
        Flag that sets the position of the icon  to the left of the text of the label (default is False)
    iconcolor : str, optional
        Color of the icon (default is 'black')
    autoselect : bool, optional
        If True, the button becomes selected when clicked (default is False)
    dark : bool, optional
        Flag to invert the text and backcolor (default is the value of settings.dark_mode)
    rounded : bool, optional
        Flag to display the button with rounded corners (default is the value of settings.button_rounded)
    colorselected : str, optional
        Color used for the button when it is selected (default is settings.color_first)
    colorunselected : str, optional
        Color used for the button when it is not selected (default is settings.color_second)
            
    Note
    ----
    All the icons from https://materialdesignicons.com/ site can be used, just by prepending 'mdi-' to their name.
    
    All the free icons from https://fontawesome.com/ site can be used, just by prepending 'fa-' to their name.
    
    
    Example
    -------
    Creation and display of a some button widgets playing with the parameters::
        
        from vois.vuetify import settings, button

        def onclick(arg=None):
            if arg==1: b1.selected = not b1.selected
            if arg==2: b2.selected = not b2.selected
            else:      b3.selected = not b3.selected

        b1 = button.button('Test button 1', textweight=300, onclick=onclick, argument=1,
                           width=150, height=36, 
                           tooltip='Tooltip for button 1', selected=False, rounded=True,
                           icon='mdi-car-light-high', iconcolor='black')

        b2 = button.button('Test button 2', textweight=450, onclick=onclick, argument=2,
                           width=150, height=48,
                           tooltip='Tooltip for button 2', selected=True, rounded=False)

        b3 = button.button('Test button 3', textweight=450, onclick=onclick, argument=3,
                           width=150, height=38,
                           textcolor=settings.color_first, 
                           tooltip='Tooltip for button 3', outlined=True, rounded=True)

        b4 = button.button('Contacts', onlytext=True, textcolor=settings.color_first,
                           width=150, height=28,
                           href='https://ec.europa.eu/info/contact_en', target="_blank",
                           tooltip='Open a URL')

        display(b1.draw())
        display(b2.draw())
        display(b3.draw())
        display(b4.draw())


    .. figure:: figures/button.png
       :scale: 100 %
       :alt: button widget

       Example of a 4 button widgets with different display modes.
   """

   
    # Initialization
    def __init__(self, text, onclick=None, argument=None, width=100, height=36, selected=False, disabled=False, tooltip='', large=False, small=False, xsmall=False, outlined=False, textweight=500,
                 href=None, target=None, onlytext=False, textcolor=None,  class_="pa-0 ma-0",
                 icon=None, iconlarge=False, iconsmall=False, iconleft=False, iconcolor='black',
                 autoselect=False, dark=settings.dark_mode, rounded=settings.button_rounded,
                 colorselected=settings.color_first, colorunselected=settings.color_second
                ):
        self.onclick    = onclick
        self.argument   = argument
        self._selected  = selected
        self._disabled  = disabled
        self.autoselect = autoselect
        self.text       = text
        self.iconlarge  = iconlarge
        self.iconsmall  = iconsmall
        self.iconcolor  = iconcolor
        self.colorselected   = colorselected
        self.colorunselected = colorunselected

        self.icondistance = " ml-2"
    
        if not textcolor is None: color = textcolor
        else:
            if self._selected: color = self.colorselected
            else:              color = self.colorunselected
            
        if icon is None:
            childs = [self.text]
        else:
            if len(self.text) == 0: self.icondistance = ""
            elif iconleft:          self.icondistance = " mr-2"
            
            icn = v.Icon(class_="pa-0 ma-0 %s" % self.icondistance, large=self.iconlarge, small=self.iconsmall, color=self.iconcolor, children=[icon])
            if iconleft:
                if len(self.text) == 0: childs = [icn]
                else:                   childs = [icn, self.text]
            else:
                if len(self.text) == 0: childs = [icn]
                else:                   childs = [self.text, icn]
            
        self.b = v.Btn(color=color, dark=dark, icon=onlytext, depressed=True, outlined=outlined, large=large, small=small, x_small=xsmall, 
                       disabled=disabled, width=width, min_width=width, height=height, min_height=height, href=href, target=target,
                       children=childs, style_='font-family: %s; font-size: 17; font-weight: %d; text-transform: none' % (fontsettings.font_name, textweight), rounded=rounded)
                
        self.b.on_event('click', self.__internal_onclick)
        
        if len(tooltip) > 0: self.b.v_on = 'tooltip.on'
        self.container = v.Container(class_=class_, children=[ v.Tooltip(color=settings.tooltip_backcolor, transition="scale-transition", bottom=True, 
                                                                         v_slots=[{'name': 'activator', 'variable': 'tooltip', 'children': self.b }],
                                                                         children=[tooltip]) ])
    
    # Returns the vuetify object to display (the v.Container)
    def draw(self):
        """Returns the ipyvuetify object to display (the internal v.Html containing a v.Btn widget as its only child)"""
        return v.Html(tag='div',children=[self.container])

    
    # Manage click event
    def __internal_onclick(self, widget=None, event=None, data=None):
        if self.onclick:
            if not self.argument is None:
                self.onclick(self.argument)
            else:
                self.onclick()
        if self.autoselect:
            self.selected = True
            

    @property
    def selected(self):
        """
        Get/Set the selected state of the button widget.
        
        Returns
        --------
        selected status : bool
            True if the button is selected, False otherwise

        Example
        -------
        Programmatically select a button::
            
            b.selected = True
            print(b.selected)
        """
        return self._selected

    @selected.setter
    def selected(self, flag):
        self._selected = bool(flag)
        if self._selected: color = self.colorselected
        else:              color = self.colorunselected
        self.b.color = color
    

    @property
    def disabled(self):
        """
        Get/Set the disabled state of the button widget.
        
        Returns
        --------
        disabled status : bool
            True if the button is disabled, False otherwise
        """
        return self._disabled

    @disabled.setter
    def disabled(self, flag):
        self._disabled = bool(flag)
        self.b.disabled = self._disabled
        
        
    # Change the icon for the button
    def setIcon(self, iconname):
        """
        Change the icon for the button

        Example
        -------
        Creation of a button and programmatically change of its icon::
                
                from vois.vuetify import settings, button
                
                b = button.button('Test button', textweight=450, width=150, height=46,
                                  selected=True, rounded=True,
                                  icon='mdi-menu-open', iconcolor='black', iconlarge=True)
                display(b.draw())
                b.setIcon('mdi-menu')

        """
        for item in self.b.children:
            if type(item).__name__ == 'Icon':
                newicon = v.Icon(class_="pa-0 ma-0 %s" % self.icondistance, large=self.iconlarge, small=self.iconsmall, color=self.iconcolor, children=[iconname])
                self.b.children = [newicon if x==item else x for x in self.b.children]
                break
                
    # Change the text for the button
    def setText(self, newtext):
        """
        Change the text for the button

        Example
        -------
        Creation of a button and programmatically change of its icon::
                
                from vois.vuetify import settings, button
                
                b = button.button('Test button', textweight=450, width=250, height=46,
                                  selected=True, rounded=True)
                display(b.draw())
                b.setText('New button text')

        """
        for item in self.b.children:
            if isinstance(item, str):
                self.b.children = [newtext if x==item else x for x in self.b.children]
                break                    


"""Extended selection of a palette of different families (sequential, divergent, etc.)"""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2022-2023
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.


# DOCUMENTATION ON MAPNIK:
# https://get-map.org/mapnik-lost-manual/book.pdf
# https://github.com/mapnik/mapnik-reference/blob/gh-pages/3.0.20/reference.json#L1517

from vois.vuetify import palettePicker, selectSingle, switch

import ipyvuetify as v
from ipywidgets import widgets
from IPython.display import display


# Utility: convert three integers to '#RRGGBB'
def RGB(r,g,b):
    return '#{:02X}{:02X}{:02X}'.format(r, g, b)
    
custompalettes = [
    { "name": "Simple", "colors": ['#000000', '#FF0000', '#00FF00', '#0000FF', '#FFFF00', '#FF00FF', '#00FFFF', '#FFFFFF']},
    
    { "name": "Dem",    "colors": [RGB(255,255,170), RGB( 39,168, 39), RGB( 11,128, 64), RGB(255,255,  0), RGB(255,186,  3),
                                   RGB(158, 30,  2), RGB(110, 40, 10), RGB(138, 94, 66), RGB(255,255,255)]},
    
    { "name": "NDVI",   "colors": [RGB(120,69,25), RGB(255,178,74), RGB(255,237,166), RGB(173,232,94),
                                   RGB(135,181,64), RGB(3,156,0), RGB(1,100,0), RGB(1,80,0)]}
]

families = ['carto', 'cmocean', 'cyclical', 'diverging', 'plotlyjs', 'qualitative', 'sequential', 'custom']



# Selection of a family and of a palette
class palettePickerEx():
    """
    Advanced selection of a palette of colors managing all the palette families and the interpolate flag
        
    Parameters
    ----------
    family : str, optional
        Family of the palette, one of these values: ['carto', 'cmocean', 'cyclical', 'diverging', 'plotlyjs', 'qualitative', 'sequential', 'custom'] (default is 'sequential')
    value : str, optional
        Name of the initially selected palette (default is 'Viridis')
    interpolate : bool, optional
        If True the colors are displayed as interpolated (default is True)
    show_interpolate_switch : bool, optional
        If True an interpolate switch is shown to enable or disable the interpolated display of the selected palette (default is True)
    width : int, optional
        Width of the widget in pixels (default is 400)
    onchange : function, optional
        Python function to call when the user selects one of the palettes. The function will as parameters the list of colors and the interpolate flag (default is None)

    Examples
    --------
    Creation of a selection widget for the palettes managing all the families::
        
        from vois.vuetify import palettePickerEx
        from ipywidgets import widgets
        from IPython.display import display

        output = widgets.Output()

        def onchange(colors, interpolate):
            with output:
                print(colors, interpolate)

        p = palettePickerEx.palettePickerEx(onchange=onchange)

        display(p)
        display(output)

    .. figure:: figures/palettePickerEx.png
       :scale: 100 %
       :alt: palettePicker widget

       Example of an extended palette picker managing all the palette families and the interpolate flag
    """
    
    # Initialization
    def __init__(self, family='sequential', value='Viridis', interpolate=True, show_interpolate_switch=True, onchange=None, width=400):
        self.family      = family
        self.interpolate = interpolate
        self.onchange    = onchange
        self.width       = width
        
        self.show_interpolate_switch = show_interpolate_switch

        self.p = None
        self.sel = selectSingle.selectSingle('Family:', families, selection=family, width=200, onchange=self.onchangeFamily, marginy=1, clearable=False)
        self.sw  = switch.switch(self.interpolate, "Interpolate", onchange=self.onchangeInterpolate)

        self.p = palettePicker.palettePicker(family=self.family, custompalettes=custompalettes, label='Palette:', width=self.width, height=26, onchange=self.onchangePalette)
        self.p.value = value

        self.spacer = v.Html(tag='div',children=[' '], style_='width: 10px;')

    
    # Draw the widget
    def draw(self):
        if self.show_interpolate_switch:
            r = widgets.HBox([self.sel.draw(), self.spacer, self.sw.draw()])
        else:
            r = self.sel.draw()
        return widgets.VBox([r, self.p.draw()])


    # Selection of a palette
    def onchangePalette(self):
        if not self.p is None:
            if not self.onchange is None:
                self.onchange(self.p.colors, self.interpolate)

    # Changed the family
    def onchangeFamily(self):
        self.family = self.sel.value
        if self.family == 'carto' or self.family == 'qualitative':
            self.interpolate = False
            self.sw.value = self.interpolate
        else:
            self.interpolate = True
            self.sw.value = self.interpolate
        self.p.updatePalettes(self.family,self.interpolate)
        self.p.value = self.p.images[0]['name']
        if not self.onchange is None:
            self.onchange(self.p.colors, self.interpolate)

    # Changed the interpolation flag
    def onchangeInterpolate(self, flag):
        self.interpolate = flag
        value = self.p.value
        oldonchange = self.onchange
        self.onchange = None
        self.p.updatePalettes(self.family,self.interpolate)
        self.onchange = oldonchange
        self.p.value = value
        if not self.onchange is None:
            self.onchange(self.p.colors, self.interpolate)

            
            
    # familyname property
    @property
    def familyname(self):
        """
        Get/Set name of the selected family.
        
        Returns
        --------
        name : str
            Name of the currently selected family

        Example
        -------
        Set and then get the current palette family::
            
            picker.familyname = 'qualitative'
            print(picker.familyname)
        
        """
        return self.sel.value
        
    
    # Select one of the families
    @familyname.setter
    def familyname(self, name):
        self.sel.value = name

        
    # value property
    @property
    def value(self):
        """
        Get/Set name of the selected palette.
        
        Returns
        --------
        name : str
            Name of the currently selected palette

        Example
        -------
        Set and then get the current palette name::
            
            picker.value = 'Viridis'
            print(picker.value)
        
        """
        return self.p.value
        
    
    # Select one of the palette given its name
    @value.setter
    def value(self, name):
        self.p.value = name
            
            
    # colors property
    @property
    def colors(self):
        """
        Get the colors of the selected palette.
        
        Returns
        --------
        colorlist : list of strings in '#RRGGBB' format
            List of colors of the selected palette

        Example
        -------
        Get the selected palette colors::
            
            print(picker.colors)
        
        """
        return self.p.colors
            


"""Input widget to select a date"""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2022-2023
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.
import ipyvuetify as v
from datetime import datetime

try:
    from . import settings
except:
    import settings


class datePicker():
    """
    Input widget to select a date.

    Parameters
    ----------
    date : str, optional
        Initial date selected on the input widget expressed in format 'YYYY-MM-DD' (default is None, corresponding to the today date)
    label : str, optional
        Label to be displayed inside the widget (default is '')
    dark : bool, optional
        If True, the popup date selection will have a dark background (default is settings.dark_mode)
    width : int, optional
        Width of the widget in pixels (default is 88)
    color : str, optional
        Color to use for the widget and the header of the popup window (default is settings.color_first)
    show_week : bool, optional
        If True the popup window will also show number of the week (default is False)
    onchange : function, optional
        Python function to call when the user selects a date. The function will receive no parameters. (default is None)
    offset_x : bool, optional
        If True the popup window will be opened on the right of the input field (default is False)
    offset_y : bool, optional
        If True the popup window will be opened on the bottom of the input field (default is True)
    disabled : bool, optional
        True if the selection of the date is disabled, False otherwise (default is False)
    mindate : str, optional
        Minimum selectable date (default is None)
    maxdate : str, optional
        Maximum selectable date (default is None)

    Example
    -------
    Creation of a date picker widget::
        
        from vois.vuetify import datePicker
        from ipywidgets import widgets
        from IPython.display import display

        output = widgets.Output()

        def onchange():
            with output:
                print('Changed to', d.date)

        d = datePicker.datePicker(date=None, label='Start date', 
                                  offset_x=True, offset_y=False,
                                  onchange=onchange)

        display(d.draw())
        display(output)

    .. figure:: figures/datePicker.png
       :scale: 100 %
       :alt: datePicker widget

       Example of a datePicker
    """
    
    def __init__(self, date=None, label='', dark=settings.dark_mode, width=88, color=settings.color_first,
                       show_week=False, onchange=None, offset_x=False, offset_y=True,
                       disabled=False, mindate=None, maxdate=None):
        
        if date is None:
            self._date = datetime.today().strftime('%Y-%m-%d')
        else:
            self._date = date

        self.label      = label
        self.dark       = dark
        self.width      = width
        self.color      = color
        self.show_week  = show_week
        self.onchange   = onchange
        self.offset_x   = offset_x
        self.offset_y   = offset_y
        self._disabled  = disabled
        self.mindate    = mindate
        self.maxdate    = maxdate

        margins = "mb-n5"
        if len(self.label) == 0: margins = "mb-n5 mt-n2"
            
        
        if self._disabled: von = ''
        else:              von = 'menuData.on'
        self.tf = v.TextField(v_on=von, style_="max-width: %dpx; overflow: hidden;"%self.width, v_model=self._date,
                              type_="date", color=self.color, readonly=True, label=self.label, class_=margins)

        self.ctf = v.Card(flat=True, style_="max-width: %dpx; overflow: hidden;"%self.width, class_="pa-0 ma-0", children=[self.tf])

        self.p = v.DatePicker(v_model=self._date, flat=True, elevation=0, color=self.color, header_color=self.color,
                              style_="max-width: 290px;", show_week=self.show_week, dark=self.dark)
        
        if not self.mindate is None: self.p.min = self.mindate
        if not self.maxdate is None: self.p.max = self.maxdate
            
        self.p.on_event('input', self.__internal_onchange)

        self.m = v.Menu(offset_x=self.offset_x, offset_y=self.offset_y, open_on_hover=False, dense=True, close_on_click=True, close_on_content_click=False,
                        v_slots=[{'name': 'activator', 'variable': 'menuData', 'children': [self.ctf]}],
                        children=[self.p] )

        
    # Manage 'input' event
    def __internal_onchange(self, widget, event, data):
        self._date = data
        self.tf.v_model = data
        if self.onchange:
            self.onchange()
        
    # Returns the vuetify object to display (the v.Menu)
    def draw(self):
        """Returns the ipyvuetify object to display (the internal v.Menu)"""
        return self.m

    
    # date property
    @property
    def date(self):
        """
        Get/Set the selected date.
        
        Returns
        --------
        d : str
            date currently selected in the format 'YYYY-MM-DD'

        Example
        -------
        Programmatically change the date::
            
            picker.date = '2022-07-15'
            print(picker.date)
        
        """
        return self._date

    
    @date.setter
    def date(self, d):
        if isinstance(d, str) and len(d) == 10 and d[4] == '-' and d[7] == '-':
            self._date = d
            self.tf.v_model = self._date
            self.p.v_model  = self._date
            if self.onchange:
                self.onchange()
        else:
            self._date = None
            self.tf.v_model = self._date
            self.p.v_model  = self._date
            
            
    # disabled property
    @property
    def disabled(self):
        """
        Get/Set the disabled state of the widget.
        
        Returns
        --------
        flag : bool
            True if the widget is disabled, False otherwise

        Example
        -------
        Programmatically change the date::
            
            picker.disabled = True
            print(picker.disabled)
        
        """
        return self._disabled

    
    @disabled.setter
    def disabled(self, flag):
        self._disabled = bool(flag)
        if self._disabled: von = ''
        else:              von = 'menuData.on'
        self.tf.v_on = von



"""Popup window opened at hover on a button."""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2022-2023
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.
from IPython.display import display
import ipyvuetify as v

try:
    from . import settings
    from . import fontsettings
    from . import tooltip
except:
    import settings
    import fontsettings
    import tooltip


#####################################################################################################################################################
# Popup control
#####################################################################################################################################################
class popup:
    """
    Popup window opened at hover on a button
        
    Parameters
    ----------
    widget : any widget
        Widget to be displayed inside the popup menu
    buttontext : str
        Text to display inside the button
    buttonwidth : int, optional
        Width of the button in pixels (default is 140)
    buttonheight : int, optional
        Height of the button in pixels (default is 40)
    icon : str, optional
        Name of the icon to be displayed inside the button (default is None)
    color : str, optional
        Color used for the button (default is the color_first defined in the settings.py module)
    rounded : bool, optional
        If True the button will be rounded (default is the button_rounded defined in the settings.py module)
    outlined : bool, optional
        If True the button will be outlined (default is True)
    text : bool, optional
        If True the button will display only the text and/or the icon, with no background (default is False)
    popupwidth : int, optional
        Width of the popup window in pixels (default is 160). The popup cannot have a width smaller than the width of the button
    popupheight : int, optional
        Height of the popup window in pixels (default is 250)
    open_on_hover : bool, optional
        If True the popup opens/closes when hovering the button, otherwise it opens/closes on click on the button (default is True)
    close_on_click : bool, optional
        Designates if popup should close on outside click (default is True)
    close_on_content_click : bool, optional
        Designates if popup should close when its content is clicked  (default is True)

    Note
    ----
    All the icons from https://materialdesignicons.com/ site can be used, just by prepending 'mdi-' to their name.
    
    All the free icons from https://fontawesome.com/ site can be used, just by prepending 'fa-' to their name.
    
    
    Example
    -------
    Creation and display of a popup window displaying a tree and opened by hovering on a button::
        
        from vois.vuetify import popup, treeview
        from IPython.display import display

        sectors = ['S%d'%x for x in range(10)]
        treecard = treeview.createTreeviewFromList(sectors,
                                                   rootName='All',
                                                   height='270px')

        p = popup.popup(treecard, 'Sectors', popupheight=270)

        display(p.draw())

    .. figure:: figures/popup.png
       :scale: 100 %
       :alt: popup widget example

       Example of a button that, on hover, opens a popup window containing a treeview
   """

    # Initialization
    def __init__(self,
                 widget,
                 buttontext,
                 buttonwidth=140,
                 buttonheight=40,
                 icon=None,
                 color=settings.color_first,
                 rounded=settings.button_rounded,
                 outlined=True,
                 text=False,
                 popupwidth=160,
                 popupheight=250,
                 open_on_hover=True,
                 close_on_click=True,
                 close_on_content_click=True):

        # The popup cannot have a width smaller than the width of the button
        if popupwidth < buttonwidth: popupwidth = buttonwidth
            
        html = v.Html(tag='div', width='%dpx'%popupwidth, height='%dpx'%popupheight, children=[widget], style_='overflow: auto;')
        card = v.Card(width='%dpx'%popupwidth, height='%dpx'%popupheight, elevation=1,
                      children=[widget], style_='overflow: auto;')
        
        children = []
        leftspace = 0
        if len(buttontext) > 0:
            children.append(buttontext)
            leftspace = 2
            
        if not icon is None:
            children.append(v.Icon(small=True, children=[icon], class_='pa-0 ma-0 ml-%d'%leftspace))
            
        self.btn = v.Btn(v_on='menuData.on', color=color, fab=False, dark=True, depressed=True, text=text,
                         disabled=False, width=buttonwidth, buttonheight=buttonheight, rounded=rounded, outlined=outlined,
                         style_='font-family: %s; font-weight: %d; text-transform: none' % (fontsettings.font_name, 450),
                         children=children)

        self.menu = v.Menu(offset_y=True, open_on_hover=open_on_hover, dense=True, close_on_click=close_on_click, close_on_content_click=close_on_content_click,
                           v_slots=[{'name': 'activator', 'variable': 'menuData', 'children': self.btn, }],
                           children=[card])
    
    # Returns the vuetify object to display
    def draw(self):
        """Returns the ipyvuetify object to display"""
        return self.menu

